require "seeding_utils/version"
require 'pathname'

module SeedingUtils

  extend self

  # Expects a proc that when called, returns a raw PG connection.
  # For example:
  #
  #     SeedingUtils.connection = -> {ActiveRecord::Base.connection.raw_connection}
  # or:
  #     SeedingUtils.connection = -> {PG::Connection.open dbname: 'foo'}
  def connect(callable)
    @connection_callable = callable
  end

  # @return PG::Connection
  def connection
    @connection_callable.call
  end

  class Table

    attr_accessor :name, :quoted_name, :prefix

    def initialize(name, prefix = '   ->')
      @conn = SeedingUtils.connection
      self.name = name
      self.quoted_name = PG::Connection.quote_ident(name)
      self.prefix = prefix
    end

    def seed(cache_file = nil, &block)
      path = Pathname(cache_file.to_s)
      return seed_uncached(&block) if cache_file.nil?
      if path.file?
        puts %{#{prefix} loading #{name} seeds from cache}
        @conn.exec %{DELETE FROM #{quoted_name}}
        load path.open 'r:binary'
      elsif block_given?
        seed_uncached(&block)
        path.open 'w:binary' do |io|
          dump(io)
        end
      end
    end

    def seed_uncached(&block)
      puts %{#{prefix} loading #{name} seeds without cache}
      yield
    end


    # Expects an open IO object for writing.
    def dump(io)
      @conn.exec %{COPY #{quoted_name} TO STDOUT WITH (FORMAT 'binary')}
      while row = @conn.get_copy_data do
        io.write(row)
      end
      io.rewind
      return io
    end

    # Expects an open IO object for reading.
    def load(io)
      @conn.exec %{DELETE FROM #{quoted_name}}
      @conn.exec %{COPY #{quoted_name} FROM STDIN WITH (FORMAT 'binary')}
      while data = io.read(256)
        @conn.put_copy_data(data)
      end
      @conn.put_copy_end
    end
  end

end
